/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
/* The following description is used by demos/make-demo-list.pl
 * to compile a list of all tests. */
/* DESCRIPTION
 * wavetable demo
 *
 * Function: wavetable_demo
 */


#include <gtk/gtk.h>
#include <gdam/gdambuffer.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "../gdamchannel.h"

typedef struct _DemoInfo {
	GdamChannel* channel;
	GtkWindow* top_level;
	GtkLabel* filename;
	GtkLabel* state;
	GtkAdjustment* hertz_adjustment;
	GtkEntry* entry;
	guint mixer_id;
	guint buffer_id;
	guint source_id;
	guint square_wave, triangle_wave, sine_wave;
	gboolean making_wave_request;
} DemoInfo;

static void indicate(DemoInfo* info, const char* state)
{
	gtk_label_set_text(info->state, state);
}

/*static void stopped_callback(gpointer user_data)
{
	DemoInfo* info = (DemoInfo*)user_data;
	if (info->wave_id == 0 || info->making_wave_request)
		return;
	info->wave_id = 0;
	indicate(info, "ended");
}*/
	
static double get_hertz(DemoInfo* info)
{

	return exp(info->hertz_adjustment->value);
}
static void play_callback(GtkButton* button, DemoInfo* info)
{
	gchar* filename;
	if (info->source_id != 0 || info->making_wave_request)
		return;
	g_return_if_fail(info->buffer_id != 0);

	info->making_wave_request = TRUE;
	gtk_label_get(info->filename, &filename);
	gdam_debug("create source");
	info->source_id = gdam_channel_create_source_generic(
				info->channel,
				"GdamWavetable",
				2,

				"buffer_id",
				GDAM_ARG_TYPE_INT,
				info->buffer_id,

				"hertz",
				GDAM_ARG_TYPE_DOUBLE,
				get_hertz(info)
	);
	g_assert(info->source_id != 0);
	gdam_debug("gdam_channel_mix_add being called...");
	if (!gdam_channel_mix_add(info->channel,
	                           info->mixer_id,
				   info->source_id,
				   -1))
		g_warning("mix_add failed");
	else
		indicate(info, "playing");
	info->making_wave_request = FALSE;
}

static void stop_callback(GtkButton* button, DemoInfo* info)
{
	if (info->source_id == 0 || info->making_wave_request)
		return;
	if (!gdam_channel_detach_source(info->channel, info->source_id))
		g_warning("detach source failed");
	else
		indicate(info, "stopped");
	info->source_id = 0;
}
static void hertz_changed(GtkAdjustment* adjustment, DemoInfo* info)
{
	double hertz = get_hertz(info);
	char buf[100];
	gdam_channel_source_set_arg(info->channel,
	                            info->source_id,
			            "hertz",
			            GDAM_ARG_TYPE_DOUBLE,
			            hertz);
	snprintf(buf, sizeof(buf), "%.5g", hertz);
	gtk_label_set_text(info->state, buf);
}

static void sine_callback(GtkButton* button, DemoInfo* info)
{
	info->buffer_id = info->sine_wave;
}
static void triangle_callback(GtkButton* button, DemoInfo* info)
{
	info->buffer_id = info->triangle_wave;
}
static void square_callback(GtkButton* button, DemoInfo* info)
{
	info->buffer_id = info->square_wave;
}
static void entry_callback(GtkButton* button, DemoInfo* info)
{
	info->buffer_id = atoi(gtk_entry_get_text(info->entry));
}

/* Helper + macro to cut down repetition in demo1_init(). */
static GtkWidget* _generic_button(const char* label, GtkSignalFunc callback,
                                 gpointer user_data)
{
	GtkWidget* button = gtk_button_new_with_label(label);
	gtk_signal_connect(GTK_OBJECT(button), "clicked", callback, user_data);
	gtk_widget_show(GTK_WIDGET(button));
	return button;
}
#define generic_button(label, callback)				\
		_generic_button(label, GTK_SIGNAL_FUNC(callback), info)

static GtkWidget* _generic_radio_button(const char* label, GtkSignalFunc cb,
                                       gpointer user_data, GSList** list)
{
	GtkWidget* button = gtk_radio_button_new_with_label(*list, label);
	gtk_signal_connect(GTK_OBJECT(button), "clicked", cb, user_data);
	gtk_widget_show(GTK_WIDGET(button));
	return button;
}
#define generic_radio_button(label, callback, plist)			\
	_generic_radio_button(label, GTK_SIGNAL_FUNC(callback), info, plist)
void wavetable_demo(GdamChannel*           channel,
                guint                  mixer_id)
{
	DemoInfo* info = g_new(DemoInfo, 1);
	GtkWidget* hbox;
	GtkWidget* vbox;
	GtkWidget* entry;
	GSList* list;
	vbox = gtk_vbox_new(FALSE, 0);
	hbox = gtk_hbox_new(FALSE, 0);

	info->channel = channel;
	info->top_level = (GtkWindow*) gtk_window_new(GTK_WINDOW_TOPLEVEL);
	info->filename = (GtkLabel*) gtk_label_new("???");
	info->state = (GtkLabel*) gtk_label_new("not started");
	info->making_wave_request = FALSE;
	info->mixer_id = mixer_id;
	info->source_id = 0;
	if ( !gdam_channel_get_server_side_int(channel,
	                                       "wavetables",
	                                       "sine",
					       &info->sine_wave)
	  || !gdam_channel_get_server_side_int(channel,
	                                       "wavetables",
	                                       "square",
					       &info->square_wave)
	  || !gdam_channel_get_server_side_int(channel,
	                                       "wavetables",
	                                       "triangle",
					       &info->triangle_wave)
	  || info->sine_wave == 0
	  || info->square_wave == 0
	  || info->triangle_wave == 0) {
	  	g_warning("error lookup for sine/square/triangle name");
		return;
	}

	g_message("sine=%d, square=%d, triangle=%d",
	       info->sine_wave, info->square_wave, info->triangle_wave);

	info->buffer_id = info->sine_wave;

	gtk_widget_show(GTK_WIDGET(info->filename));
	gtk_box_pack_start_defaults(GTK_BOX(hbox), GTK_WIDGET(info->filename));
	gtk_widget_show(hbox);
	gtk_box_pack_start_defaults(GTK_BOX(hbox), GTK_WIDGET(info->state));
	gtk_widget_show(GTK_WIDGET(info->state));

	gtk_box_pack_start_defaults(GTK_BOX(vbox), GTK_WIDGET(hbox));

	hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start_defaults(GTK_BOX(hbox),
	                        generic_button("Play", play_callback));
	gtk_box_pack_start_defaults(GTK_BOX(hbox),
	                        generic_button("Stop", stop_callback));
	gtk_widget_show(hbox);
	gtk_box_pack_start_defaults(GTK_BOX(vbox), GTK_WIDGET(hbox));

	info->hertz_adjustment = (GtkAdjustment*)gtk_adjustment_new(
						log(440), log(1), log(30000),
						1, 1, 1);
	gtk_signal_connect(GTK_OBJECT(info->hertz_adjustment),
	                   "value_changed",
			   GTK_SIGNAL_FUNC(hertz_changed),
			   info);
	{
		GtkWidget* bar = gtk_hscrollbar_new(info->hertz_adjustment);
		gtk_widget_show(bar);
		gtk_box_pack_start_defaults(GTK_BOX(vbox), bar);
	}

	hbox = gtk_hbox_new(FALSE, 0);
	list = NULL;
	gtk_box_pack_start_defaults(GTK_BOX(hbox),
		generic_radio_button("sin", sine_callback, &list));
	gtk_box_pack_start_defaults(GTK_BOX(hbox),
		generic_radio_button("square", square_callback, &list));
	gtk_box_pack_start_defaults(GTK_BOX(hbox),
		generic_radio_button("triangle", triangle_callback, &list));
	gtk_box_pack_start_defaults(GTK_BOX(hbox),
		generic_radio_button("custom...", entry_callback, &list));

	entry = gtk_entry_new();
	gtk_widget_show(entry);
	gtk_box_pack_start_defaults(GTK_BOX(hbox), entry);
	info->entry = GTK_ENTRY(entry);

	gtk_widget_show(hbox);
	gtk_box_pack_start_defaults(GTK_BOX(vbox), GTK_WIDGET(hbox));

	gtk_widget_show(vbox);
	gtk_container_add(GTK_CONTAINER(info->top_level), GTK_WIDGET(vbox));
	gtk_widget_show(GTK_WIDGET(info->top_level));
}
