/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAM_RECORDER_H_
#define __GDAM_RECORDER_H_


typedef struct _GdamRecorder GdamRecorder;
typedef struct _GdamRecorderClass GdamRecorderClass;
#include <gdam/gdamfilter.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "../gdamcachebuffer.h"

GtkType gdam_recorder_get_type();
#define GDAM_TYPE_RECORDER			(gdam_recorder_get_type ())
#define GDAM_RECORDER(obj)              (GTK_CHECK_CAST ((obj), GDAM_TYPE_RECORDER, GdamRecorder))
#define GDAM_RECORDER_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAM_TYPE_RECORDER, GdamRecorderClass))
#define GDAM_IS_RECORDER(obj)           (GTK_CHECK_TYPE ((obj), GDAM_TYPE_RECORDER))
#define GDAM_IS_RECORDER_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAM_TYPE_RECORDER))

struct _GdamRecorderClass {
	GdamFilterClass		recorder_class;
};
struct _GdamRecorder {
	GdamFilter		recorder;
	GdamBuffer		buffer;
	gboolean		is_recording;
};

/* Stop recording. */
void gdam_recorder_stop              (GdamRecorder*  recorder);

/* Start recording. */
void gdam_recorder_start             (GdamRecorder*  recorder);

/* Drain the buffer. */
void gdam_recorder_empty             (GdamRecorder*  recorder);

/* Make a buffer out of the source, optionally removing the 
 * data from this buffer at the same time. */
GdamCacheBuffer* gdam_recorder_make_buffer
                                     (GdamRecorder*  recorder,
                                      gboolean       drain);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
