/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/

#include "gdamsynthdelayline.h"

static void gdam_synth_delay_line_init(GdamSynthDelayLine* synth_delay_line)
{
}
static void gdam_synth_delay_line_class_init(GdamSynthDelayLineClass* synth_delay_line_class)
{
}

GtkType gdam_synth_delay_line_get_type()
{
	static GtkType synth_delay_line_type = 0;
	if (!synth_delay_line_type) {
		static const GtkTypeInfo synth_delay_line_info = {
			"GdamSynthDelayLine",
			sizeof(GdamSynthDelayLine),
			sizeof(GdamSynthDelayLineClass),
			(GtkClassInitFunc) gdam_synth_delay_line_class_init,
			(GtkObjectInitFunc) gdam_synth_delay_line_init,
			/* reserved_1*/ NULL,
			/* reserved_2*/ NULL,
			(GtkClassInitFunc) NULL
		};
		synth_delay_line_type = gtk_type_unique(GDAM_TYPE_SYNTH_FILTER,
		                                 &synth_delay_line_info);
	}
	return synth_delay_line_type;
}

void      gdam_synth_delay_line_set_buffer_length
                               (GdamSynthDelayLine* delay,
                                int                 max_buffer_length)
{
	if (delay->buffer == NULL) {
		int i;
		delay->buffer = g_new(gdouble, max_buffer_length);
		for (i = 0; i < max_buffer_length; i++)
			delay->buffer[i] = 0.0;
	} else {
		int amount_buffered = delay->in_point - delay->out_point;
		int i;
		gdouble* new_buf = g_new(gdouble, max_buffer_length);
		if (amount_buffered < 0)
			amount_buffered += delay->buffer_length;
		if (amount_buffered < max_buffer_length) {
			/* We can keep all buffered data. */
			int p1 = delay->buffer_length - delay->out_point;
			int p2 = amount_buffered - p1;
			if (p2 > 0) {
				memcpy(new_buf,
				       delay->buffer + delay->out_point,
				       p1);
				memcpy(new_buf + p1,
				       delay->buffer,
				       p2);
			} else {
				memcpy(new_buf,
				       delay->buffer + delay->out_point,
				       amount_buffered);
			}
			delay->in_point = amount_buffered;
			delay->out_point = 0;
		} else {
			/* We keep only max_buffer_length - 1 samples. */
			int to_preserve = max_buffer_length - 1;
			int p1 = delay->buffer_length - delay->out_point;
			int p2 = to_preserve - p1;
			if (p2 > 0) {
				memcpy(new_buf,
				       delay->buffer + delay->out_point,
				       p1);
				memcpy(new_buf + p1,
				       delay->buffer,
				       p2);
			} else {
				memcpy(new_buf,
				       delay->buffer + delay->out_point,
				       to_preserve);
			}
			delay->in_point = to_preserve;
			delay->out_point = 0;
		}
		for (i = amount_buffered; i < max_buffer_length; i++)
			new_buf[i] = 0.0;
		g_free(delay->buffer);
		delay->buffer = new_buf;
	}
	delay->buffer_length = max_buffer_length;
}
